package Business::TrueLayer::Webhook::Mandate;

use strict;
use warnings;
use feature qw/ signatures postderef /;

no warnings qw/ experimental::signatures experimental::postderef /;

use Moose;
extends 'Business::TrueLayer::Webhook::Resource';
use namespace::autoclean;

=head1 SEE ALSO

L<Business::TrueLayer::Webhook::Resource>

=cut

1;
